"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const program = require("commander");
const inquirer = require("inquirer");
const config = require('../resources/config.json');
const chalk_1 = require("chalk");
const birthday_1 = require("./birthday");
const config_1 = require("./utils/config");
const pkg = require('../package.json');
const loginText = '\nPlease wait! Logging & Fetching Birthdays!...\n';
program.version(pkg.version)
    .description('Facebook Birthday CLI')
    .command('*', '', { noHelp: true, isDefault: true })
    .action(() => {
    program.help();
});
program.command('wish')
    .alias('w')
    .description('Facebook wish command')
    .option('-a, --all', 'wish all friends')
    .option('-r, --reset', 'resets all config values to default')
    .action((options) => __awaiter(this, void 0, void 0, function* () {
    try {
        const today = new Date().getDay();
        if (options.reset) {
            config.firstLogin = true;
            config.save = false;
            const keys = Object.keys(config);
            for (let i = 2; i < 5; i++) {
                config[keys[i]] = '';
            }
            config.day = 1;
            config.birthdayNames = [];
            yield config_1.writeFile(config);
            console.log(chalk_1.default.green('\nSuccessfully reset to default values! Please run "facebook wish" to start over.\n'));
            process.exit(0);
        }
        const Wisher = new birthday_1.BirthdayWisher();
        let credentials;
        if (config.firstLogin === true) {
            console.log('\n');
            config.day = today;
            yield config_1.writeFile(config);
            credentials = yield inquirer.prompt([
                {
                    message: 'Please enter your facebook username:',
                    name: 'username',
                    type: 'input',
                },
                {
                    message: 'Please enter your facebook password:',
                    name: 'password',
                    mask: '*',
                    type: 'password',
                },
            ]);
            console.log(chalk_1.default.yellowBright(loginText));
            yield Wisher.login(credentials);
        }
        else {
            if (config.day === today && config.birthdayNames.length === 0) {
                console.error(chalk_1.default.red('\n' +
                    'You have wished all your friends, Please try tomorrow!' +
                    '\n'));
                process.exit(0);
            }
            const savedCredentials = {};
            config_1.decryptCredentials(savedCredentials);
            console.log(chalk_1.default.yellowBright(loginText));
            yield Wisher.login(savedCredentials);
        }
        if (!options.all) {
            yield Wisher.findAndWish(credentials);
        }
        else {
            if (!config.save) {
                birthday_1.wishes.splice(birthday_1.wishes.indexOf('custom message'), 1);
                const answer = yield inquirer.prompt([
                    {
                        choices: birthday_1.wishes,
                        message: 'Select your favourite birthday wish:',
                        name: 'wish',
                        type: 'list',
                    },
                    {
                        default: true,
                        message: 'Do you want to save this configuration?',
                        name: 'config',
                        type: 'confirm',
                    },
                ]);
                if (answer.config) {
                    config.save = true;
                    config.wish = answer.wish;
                    yield config_1.writeFile(config);
                }
                yield Wisher.findAndWishAll(credentials, answer.wish);
            }
            else {
                yield Wisher.findAndWishAll(credentials, config.wish);
            }
        }
    }
    catch (Exception) {
        console.error(chalk_1.default.red('\n' + Exception.toString() + '\n'));
        process.exit(0);
    }
}));
program.parse(process.argv);
if (!program.args.length) {
    program.help();
}
//# sourceMappingURL=cli.js.map