"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const puppeteer_1 = require("puppeteer");
const locators_1 = require("./locators");
class PageHelper {
    constructor() {
        this.retryCount = 2;
        this.browser = null;
        this.page = null;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.browser = yield puppeteer_1.launch({
                headless: true,
                args: ['--disable-notifications', '--start-maximized'],
            });
            this.page = yield this.browser.newPage();
        });
    }
    /**
     * @param  {string} url
     * @returns Promise
     */
    open(url) {
        return __awaiter(this, void 0, void 0, function* () {
            let i = 0;
            while (i < this.retryCount) {
                try {
                    return yield this.page.goto(url);
                }
                catch (Exception) {
                    i++;
                    if (i === this.retryCount) {
                        throw Exception;
                    }
                }
            }
        });
    }
    /**
     * @returns Promise
     */
    getTitle() {
        return __awaiter(this, void 0, void 0, function* () {
            let i = 0;
            while (i < this.retryCount) {
                try {
                    return yield this.page.title();
                }
                catch (Exception) {
                    i++;
                    if (i === this.retryCount) {
                        throw Exception;
                    }
                }
            }
        });
    }
    /**
     * @param  {string} element
     * @returns Promise
     */
    $$(element) {
        return __awaiter(this, void 0, void 0, function* () {
            let i = 0;
            while (i < this.retryCount) {
                try {
                    return yield this.page.$$(element);
                }
                catch (Exception) {
                    i++;
                    if (i === this.retryCount) {
                        throw Exception;
                    }
                }
            }
        });
    }
    /**
     * @param  {string} element
     * @returns Promise
     */
    focusElement(element) {
        return __awaiter(this, void 0, void 0, function* () {
            let i = 0;
            while (i < this.retryCount) {
                try {
                    yield this.page.waitForSelector(element);
                    return yield this.page.focus(element);
                }
                catch (Exception) {
                    try {
                        i++;
                        yield this.page.waitForSelector(element);
                        yield this.page.evaluate('arguments[0].click()', element);
                        continue;
                    }
                    catch (Exception) {
                        i++;
                        if (i === this.retryCount) {
                            throw Exception;
                        }
                    }
                }
            }
        });
    }
    /**
     * @param  {string} element
     * @returns Promise
     */
    clickElement(element) {
        return __awaiter(this, void 0, void 0, function* () {
            let i = 0;
            while (i < this.retryCount) {
                try {
                    yield this.page.waitForSelector(element);
                    return yield this.page.click(element);
                }
                catch (Exception) {
                    try {
                        i++;
                        yield this.page.waitForSelector(element);
                        yield this.page.evaluate('arguments[0].click()', element);
                        continue;
                    }
                    catch (Exception) {
                        i++;
                        if (i === this.retryCount) {
                            throw Exception;
                        }
                    }
                }
            }
        });
    }
    /**
     * @param  {string} element
     * @returns Promise
     */
    xpathClick(element) {
        return __awaiter(this, void 0, void 0, function* () {
            let i = 0;
            while (i < this.retryCount) {
                try {
                    yield this.page.waitForXPath(element);
                    const xpathElement = yield this.page.$x(element);
                    return yield xpathElement[0].click();
                }
                catch (Exception) {
                    try {
                        i++;
                        yield this.page.waitForXPath(element);
                        yield this.page.evaluate('arguments[0].click()', element);
                        continue;
                    }
                    catch (Exception) {
                        i++;
                        if (i === this.retryCount) {
                            throw Exception;
                        }
                    }
                }
            }
        });
    }
    /**
     * @param  {string} element
     * @returns Promise
     */
    clearElement(element) {
        return __awaiter(this, void 0, void 0, function* () {
            let i = 0;
            while (i < this.retryCount) {
                try {
                    return yield this.page.type(element, '');
                }
                catch (Exception) {
                    try {
                        i++;
                        yield this.page.waitForSelector(element);
                        yield this.page.evaluate('arguments[0].click()', element);
                        continue;
                    }
                    catch (Exception) {
                        i++;
                        if (i === this.retryCount) {
                            throw Exception;
                        }
                    }
                }
            }
        });
    }
    /**
     * @param  {string} element
     * @param  {string} text
     * @returns Promise
     */
    sendElementText(element, text) {
        return __awaiter(this, void 0, void 0, function* () {
            let i = 0;
            while (i < this.retryCount) {
                try {
                    return yield this.page.type(element, text);
                }
                catch (Exception) {
                    try {
                        i++;
                        yield this.page.waitForSelector(element);
                        yield this.page.evaluate('arguments[0].click()', element);
                        continue;
                    }
                    catch (Exception) {
                        i++;
                        if (i === this.retryCount) {
                            throw Exception;
                        }
                    }
                }
            }
        });
    }
    /**
     * @param  {string} keys
     * @returns Promise
     */
    enterKeys(keys) {
        return __awaiter(this, void 0, void 0, function* () {
            let i = 0;
            while (i < this.retryCount) {
                try {
                    return yield this.page.keyboard.press(keys);
                }
                catch (Exception) {
                    i++;
                    if (i === this.retryCount) {
                        throw Exception;
                    }
                }
            }
        });
    }
    /**
     * @param  {ElementHandle} element
     * @param  {any} answers
     * @returns Promise
     */
    enterText(element, text) {
        return __awaiter(this, void 0, void 0, function* () {
            yield element.focus();
            yield element.type('');
            yield element.type(text);
            yield this.enterKeys('Enter');
            yield this.page.waitFor(500);
        });
    }
    /**
     * @param  {ElementHandle[]} elements
     * @param  {any} answers
     * @returns Promise
     */
    enterAllElementText(elements, text) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (elements.length > 0) {
                    for (let i = 0; i < elements.length; i++) {
                        yield this.enterText(elements[i], text);
                    }
                }
                else {
                    throw new Error('No textboxes present!');
                }
            }
            catch (Exception) {
                console.error(chalk_1.default.red('\nFailed to wish! Most likely you have already wished your friends or due to below reason:'));
                throw Exception;
            }
        });
    }
    /**
     * @param  {number} index
     * @param  {ElementHandle[]} elements
     * @param  {string} text
     * @returns Promise
     */
    enterElementText(index, elements, text) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (elements.length > 0) {
                    return yield this.enterText(elements[index], text);
                }
                else {
                    throw new Error('No textboxes present!');
                }
            }
            catch (Exception) {
                console.error(chalk_1.default.red('\nFailed to wish! Most likely you have already wished your friend or due to below reason:'));
                throw Exception;
            }
        });
    }
    /**
     * @param  {string} element
     * @returns Promise
     */
    getText(element) {
        return __awaiter(this, void 0, void 0, function* () {
            let i = 0;
            while (i < this.retryCount) {
                try {
                    return yield this.page.evaluate((element) => {
                        const els = Array.from(document.querySelectorAll(element));
                        return els.map((el) => {
                            return el.textContent;
                        });
                    }, element);
                }
                catch (Exception) {
                    try {
                        i++;
                        yield this.page.waitForSelector(element);
                        yield this.page.evaluate('arguments[0].click()', element);
                        continue;
                    }
                    catch (Exception) {
                        i++;
                        if (i === this.retryCount) {
                            throw Exception;
                        }
                    }
                }
            }
        });
    }
    /**
     * @param  {any} options?
     * @returns Promise
     */
    waitForNavigation(options) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.page.waitForNavigation(options);
            }
            catch (Exception) {
                throw new Error(Exception.toString());
            }
        });
    }
    /**
     * @param  {string} element
     * @returns Promise
     */
    isElementExists(element) {
        return __awaiter(this, void 0, void 0, function* () {
            let i = 0;
            while (i < this.retryCount) {
                try {
                    yield this.page.waitForSelector(element, { timeout: 1000 });
                    if ((yield this.page.$(element)) !== null) {
                        return true;
                    }
                    else {
                        return false;
                    }
                }
                catch (Exception) {
                    i++;
                    if (i === this.retryCount) {
                        return false;
                    }
                }
            }
        });
    }
    /**
     * @returns Promise
     */
    logout() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.clickElement(locators_1.locators.logoutLink);
                yield this.xpathClick(locators_1.locators.logoutButton);
                yield this.page.waitFor(500);
            }
            catch (Exception) {
                console.error('\nLogout Failed! Most likely due to below reason');
                throw Exception;
            }
        });
    }
}
exports.PageHelper = PageHelper;
//# sourceMappingURL=pageHelper.js.map